@@ -41,12 +41,18 @@ class Dense(nn.Module):
     def get_sentence_embedding_dimension(self) -> int:
         return self.out_features
 
+    def get_config_dict(self):
+        return {'in_features': self.in_features, 'out_features': self.out_features, 'bias': self.bias, 'activation_function': fullname(self.activation_function)}
+
     def save(self, output_path):
         with open(os.path.join(output_path, 'config.json'), 'w') as fOut:
-            json.dump({'in_features': self.in_features, 'out_features': self.out_features, 'bias': self.bias, 'activation_function': fullname(self.activation_function)}, fOut)
+            json.dump(self.get_config_dict(), fOut)
 
         torch.save(self.state_dict(), os.path.join(output_path, 'pytorch_model.bin'))
 
+    def __repr__(self):
+        return "Dense({})".format(self.get_config_dict())
+
     @staticmethod
     def load(input_path):
         with open(os.path.join(input_path, 'config.json')) as fIn:
