@@ -13,7 +13,7 @@ class InputExample:
         :param guid
             id for the example
         :param texts
-            the texts for the example. Note, str.strip() is called on the texts
+            the texts for the example.
         :param label
             the label for the example
         """
@@ -22,4 +22,4 @@ class InputExample:
         self.label = label
 
     def __str__(self):
-        return "<InputExample> label: {}, texts: {}".format(str(self.label), "; ".join(self.texts))
\ No newline at end of file
+        return "<InputExample> label: {}, texts: {}".format(str(self.label), "; ".join(self.texts))
